@@ -1,7 +1,7 @@
-const char actions_rcs[] = "$Id: actions.c,v 1.32.2.4 2003/12/03 10:33:11 oes Exp $";
+const char actions_rcs[] = "$Id: actions.c,v 1.35 2006/07/18 14:48:45 david__schmidt Exp $";
 /*********************************************************************
  *
- * File        :  $Source: /cvsroot/ijbswa/current/Attic/actions.c,v $
+ * File        :  $Source: /cvsroot/ijbswa/current/actions.c,v $
  *
  * Purpose     :  Declares functions to work with actions files
  *                Functions declared include: FIXME
@@ -33,6 +33,17 @@ const char actions_rcs[] = "$Id: actions.c,v 1.32.2.4 2003/12/03 10:33:11 oes Ex
  *
  * Revisions   :
  *    $Log: actions.c,v $
+ *    Revision 1.35  2006/07/18 14:48:45  david__schmidt
+ *    Reorganizing the repository: swapping out what was HEAD (the old 3.1 branch)
+ *    with what was really the latest development (the v_3_0_branch branch)
+ *
+ *    Revision 1.32.2.6  2006/01/29 23:10:56  david__schmidt
+ *    Multiple filter file support
+ *
+ *    Revision 1.32.2.5  2005/06/09 01:18:41  david__schmidt
+ *    Tweaks to conditionally include pthread.h if FEATURE_PTHREAD is enabled -
+ *    this becomes important when jcc.h gets included later down the line.
+ *
  *    Revision 1.32.2.4  2003/12/03 10:33:11  oes
  *    - Implemented Privoxy version requirement through
  *      for-privoxy-version= statement in {{settings}}
@@ -201,6 +212,10 @@ const char actions_rcs[] = "$Id: actions.c,v 1.32.2.4 2003/12/03 10:33:11 oes Ex
 #include <assert.h>
 #include <stdlib.h>
 
+#ifdef FEATURE_PTHREAD
+#include <pthread.h>
+#endif
+
 #include "project.h"
 #include "jcc.h"
 #include "list.h"
@@ -871,7 +886,7 @@ void free_current_action (struct current_action_spec *src)
 }
 
 
-static struct file_list *current_actions_file[MAX_ACTION_FILES]  = {
+static struct file_list *current_actions_file[MAX_AF_FILES]  = {
    NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL
 };
@@ -894,7 +909,7 @@ void unload_current_actions_file(void)
 {
    int i;
 
-   for (i = 0; i < MAX_ACTION_FILES; i++)
+   for (i = 0; i < MAX_AF_FILES; i++)
    {
       if (current_actions_file[i])
       {
@@ -979,7 +994,7 @@ int load_actions_file(struct client_state *csp)
    int i;
    int result;
 
-   for (i = 0; i < MAX_ACTION_FILES; i++)
+   for (i = 0; i < MAX_AF_FILES; i++)
    {
       if (csp->config->actions_file[i])
       {
