@@ -1,7 +1,7 @@
-const char miscutil_rcs[] = "$Id: miscutil.c,v 1.37.2.4 2003/12/01 14:45:14 oes Exp $";
+const char miscutil_rcs[] = "$Id: miscutil.c,v 1.44 2006/11/07 12:46:43 fabiankeil Exp $";
 /*********************************************************************
  *
- * File        :  $Source: /cvsroot/ijbswa/current/Attic/miscutil.c,v $
+ * File        :  $Source: /cvsroot/ijbswa/current/miscutil.c,v $
  *
  * Purpose     :  zalloc, hash_string, safe_strerror, strcmpic,
  *                strncmpic, chomp, and MinGW32 strdup
@@ -36,6 +36,40 @@ const char miscutil_rcs[] = "$Id: miscutil.c,v 1.37.2.4 2003/12/01 14:45:14 oes
  *
  * Revisions   :
  *    $Log: miscutil.c,v $
+ *    Revision 1.44  2006/11/07 12:46:43  fabiankeil
+ *    Silence compiler warning on NetBSD 3.1.
+ *
+ *    Revision 1.43  2006/09/23 13:26:38  roro
+ *    Replace TABs by spaces in source code.
+ *
+ *    Revision 1.42  2006/09/09 14:01:45  fabiankeil
+ *    Integrated Oliver Yeoh's domain pattern fix
+ *    to make sure *x matches xx. Closes Patch 1217393
+ *    and Bug 1170767.
+ *
+ *    Revision 1.41  2006/08/18 16:03:17  david__schmidt
+ *    Tweak for OS/2 build happiness.
+ *
+ *    Revision 1.40  2006/08/17 17:15:10  fabiankeil
+ *    - Back to timegm() using GnuPG's replacement if necessary.
+ *      Using mktime() and localtime() could add a on hour offset if
+ *      the randomize factor was big enough to lead to a summer/wintertime
+ *      switch.
+ *
+ *    - Removed now-useless Privoxy 3.0.3 compatibility glue.
+ *
+ *    - Moved randomization code into pick_from_range().
+ *
+ *    - Changed parse_header_time definition.
+ *      time_t isn't guaranteed to be signed and
+ *      if it isn't, -1 isn't available as error code.
+ *      Changed some variable types in client_if_modified_since()
+ *      because of the same reason.
+ *
+ *    Revision 1.39  2006/07/18 14:48:46  david__schmidt
+ *    Reorganizing the repository: swapping out what was HEAD (the old 3.1 branch)
+ *    with what was really the latest development (the v_3_0_branch branch)
+ *
  *    Revision 1.37.2.4  2003/12/01 14:45:14  oes
  *    Fixed two more problems with wildcarding in simplematch()
  *
@@ -229,6 +263,10 @@ const char miscutil_rcs[] = "$Id: miscutil.c,v 1.37.2.4 2003/12/01 14:45:14 oes
 #include <ctype.h>
 #include <assert.h>
 
+#ifndef HAVE_TIMEGM
+#include <time.h>
+#endif /* #ifndef HAVE_TIMEGM */
+
 #include "project.h"
 #include "miscutil.h"
 #include "errlog.h"
@@ -735,7 +773,7 @@ char *string_toupper(const char *string)
 
    while (*q != '\0')
    {
-      *p++ = toupper(*q++);
+      *p++ = toupper((int) *q++);
    }
 
    return result;
@@ -853,10 +891,21 @@ int simplematch(char *pattern, char *text)
       }
       else if (pat != fallback)
       {
+         /*
+          * Increment text pointer if in char range matching
+          */
+         if (*pat == ']')
+         {
+            txt++;
+         }
          /*
           * Wildcard mode && nonmatch beyond fallback: Rewind pattern
           */
          pat = fallback;
+         /*
+          * Restart matching from current text pointer
+          */
+         continue;
       }
       txt++;
    }
@@ -1017,6 +1066,98 @@ char * make_path(const char * dir, const char * file)
 }
 
 
+/*********************************************************************
+ *
+ * Function    :  pick_from_range
+ *
+ * Description :  Pick a positive number out of a given range.
+ *                Should only be used if randomness would be nice,
+ *                but isn't really necessary.
+ *
+ * Parameters  :
+ *          1  :  range: Highest possible number to pick.
+ *
+ * Returns     :  Picked number. 
+ *
+ *********************************************************************/
+
+long int pick_from_range(long int range)
+{
+   long int number;
+#ifndef HAVE_RANDOM
+   unsigned int weak_seed;
+
+   weak_seed = (unsigned int)((unsigned int)time(NULL) | (unsigned int)range);
+   srand(weak_seed);
+   /*
+    * Some rand implementations aren't that random and return mostly
+    * lower numbers. Low entropy doesn't matter for the header times, 
+    * but higher "random" factors are prefered.
+    */
+   number = (rand() * 12345) % (long int)(range + 1);
+   /* Overflows don't matter either, positive numbers do. */
+   if(number<0)
+   {
+      number*= -1;
+   }
+#else
+   number = random() % range + 1; 
+#endif /* (ifndef HAVE_RANDOM) */
+   return (number);
+}
+
+
+#ifndef HAVE_TIMEGM
+/*********************************************************************
+ *
+ * Function    :  timegm
+ *
+ * Description :  libc replacement function for the inverse of gmtime()
+ *                Copyright (C) 2004 Free Software Foundation, Inc.
+ *                Code copied from GnuPG with minor style changes.
+ *
+ * Parameters  :
+ *          1  :  tm: Broken-down time struct.
+ *
+ * Returns     :  tm converted into time_t seconds. 
+ *
+ *********************************************************************/
+
+time_t timegm(struct tm *tm)
+{
+   time_t answer;
+   char *zone;
+
+   zone=getenv("TZ");
+   putenv("TZ=UTC");
+   tzset();
+   answer=mktime(tm);
+   if(zone)
+   {
+      char *old_zone;
+
+      old_zone=malloc(3+strlen(zone)+1);
+      if(old_zone)
+      {
+         strcpy(old_zone,"TZ=");
+         strcat(old_zone,zone);
+         putenv(old_zone);
+      }
+   }
+   else
+   {
+#ifdef HAVE_UNSETENV
+      unsetenv("TZ");
+#else
+      putenv("TZ");
+#endif
+   }
+   tzset();
+   return answer;
+}
+#endif /* (ifndef HAVE_TIMEGM) */
+
+
 /*
  * What follows is a portable snprintf routine, written by Mark Martinec.
  * See: http://www.ijs.si/software/snprintf/
